package com.stellvoy.customer.service.serviceImpl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.stellvoy.api.client.user.UserClient;
import com.stellvoy.api.dto.user.NewUserDTO;
import com.stellvoy.api.dto.user.UserDTO;
import com.stellvoy.customer.entity.*;
import com.stellvoy.customer.mapper.sessionMapper;
import com.stellvoy.customer.service.sessionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.net.MalformedURLException;
import java.net.URL;
import java.time.LocalDateTime;
import java.util.*;

@Service
public class sessionServiceImpl extends ServiceImpl<sessionMapper, sessionEntity> implements sessionService {
    @Autowired
    private sessionMapper sessionMapper;
    @Autowired
    private UserClient userClient;

    /**
     * 根据用户ID获取会话信息。
     *
     * @param userId 用户ID
     * @return 包含会话信息和消息的Map
     */
    public Map<String, Object> getSessionByUserId(Long userId,String userName) {
        Map<String, Object> result = new HashMap<>();

        try {
            if (userId == null) {
                result.put("error", "用户ID不能为空");
                return result;
            }

            sessionEntity session = sessionMapper.findSessionByUserId(userId);
            if (session == null) {
                // 创建一条新的会话记录
                session = new sessionEntity();
                session.setUserId(userId);
                session.setNickName(userName);
                session.setCreateTime(LocalDateTime.now());
                sessionMapper.insertSession(session);

                // 创建一条新的消息记录
                session = sessionMapper.findSessionByUserId(userId);
                messageEntity message = new messageEntity();
                message.setSenderId(0L); // 假设系统发送者的ID为0
                message.setMessageContent("亲，请问有什么可以帮到您呢？");
                message.setMessageStatus(1L);
                message.setSessionId(session.getSessionId());
                message.setCreateTime(LocalDateTime.now());
                sessionMapper.insertMessage(message);

                result.put("message", "亲，请问有什么可以帮到您呢？");
                List<sessionEntity> sessions = sessionMapper.getSessionWithMessagesByUserId(userId);
                result.put("sessions", sessions);
            } else {
                List<sessionEntity> sessions = sessionMapper.getSessionWithMessagesByUserId(userId);
                result.put("sessions", sessions);
            }
            NewUserDTO user = userClient.queryUserById(userId);
            if (user== null) {
                throw new IllegalArgumentException("该用户已被禁用");
            }

            result.put("user", user);
            return result;
        } catch (Exception e) {
            result.put("error", "获取会话信息失败: " + e.getMessage());
            return result;
        }
    }

    /**
     * 根据当前登录人的ID获取文件信息。
     * @param userId
     * @return
     */
    @Override
    public List<sessionEntity> getFileByUserId(Long userId, String filePath, Long fileType,String fileName) {
        if (filePath == null || userId == null || fileType == null) {
            return Collections.emptyList();
        }

        sessionEntity session = sessionMapper.findSessionByUserId(userId);
        Long sessionId = null;
        if (session != null) {
            sessionId = session.getSessionId();
        }

        fileEntity file = new fileEntity();
        messageEntity message = new messageEntity();

        // 提取文件名
        try {
            URL url = new URL(filePath);
            String path = url.getPath(); // 获取URL路径部分
            String fileNam = path.substring(path.lastIndexOf('/') + 1); // 提取最后一个斜杠后的部分

            if (userId != 0) {
                file.setSessionId(sessionId);
                file.setFilePath(filePath);
                file.setFileType(fileType);
                file.setCreateTime(LocalDateTime.now());
                file.setFileName(fileNam);
                sessionMapper.insertFileData(file);

                message.setMessageContent(fileName);
                message.setSessionId(sessionId);
                message.setSenderId(userId);
                message.setCreateTime(LocalDateTime.now());
                sessionMapper.insertMessage(message);
            }

        } catch (MalformedURLException e) {
            // 处理URL解析错误
            System.err.println("Invalid URL: " + e.getMessage());
            return Collections.emptyList();
        }

        return null; // 或者返回其他合适的值
    }

}

