package com.stellvoy.user.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.stellvoy.api.client.auth.AuthClient;
import com.stellvoy.api.client.task.TaskClient;
import com.stellvoy.api.client.user.UserClient;
import com.stellvoy.api.dto.task.NewTaskDTO;
import com.stellvoy.api.dto.user.LoginFormDTO;
import com.stellvoy.common.domain.dto.LoginUserDTO;
import com.stellvoy.common.exceptions.BadRequestException;
import com.stellvoy.common.exceptions.ForbiddenException;
import com.stellvoy.common.utils.AssertUtils;
import com.stellvoy.common.utils.BeanUtils;
import com.stellvoy.common.utils.StringUtils;
import com.stellvoy.common.utils.UserContext;
import com.stellvoy.user.domain.dto.*;
import com.stellvoy.user.domain.po.User;
import com.stellvoy.user.domain.po.UserTask;
import com.stellvoy.user.mapper.UserMapper;
import com.stellvoy.user.service.ICodeService;
import com.stellvoy.user.service.IUserService;
import com.stellvoy.user.service.UserTaskService;
import com.stellvoy.user.utils.AliyunIdCardVerifier;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.StringHttpMessageConverter;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;


import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static com.stellvoy.user.constants.UserErrorInfo.Msg.*;


/**
 * <p>
 * 学员用户表 服务实现类
 * </p>
 */
@Service
@Slf4j
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements IUserService {

    @Autowired
    private ICodeService codeService;
    @Autowired
    private AliyunIdCardVerifier aliyunIdCardVerifier;
    @Autowired
    private StringRedisTemplate stringRedisTemplate;
    @Autowired
    private UserTaskService userTaskService;
    @Autowired
    private TaskClient taskClient;
    @Autowired
    private AuthClient authClient;
    @Value("${wechat.appid}")
    private String appId;

    @Value("${wechat.appsecret}")
    private String appSecret;
    private final RestTemplate restTemplate = new RestTemplate();

    @Override
    public LoginUserDTO queryUserDetail(LoginFormDTO loginDTO, boolean isStaff) {
        // 1.判断登录方式
        Integer type = loginDTO.getType();
        User user = null;
        // 2.手机号和密码登录
        if (type == 1) {
            user = loginByPw(loginDTO);
        }
        // 3.验证码登录
        if (type == 2) {
            user = loginByVerifyCode(loginDTO.getCellPhone(), loginDTO.getPassword());
        }
        // 微信登录
        if (type == 3) {
            int userId = Integer.parseInt(loginDTO.getUsername());
            user = lambdaQuery().eq(User::getId, userId).one();
        }
        // 4.错误的登录方式
        if (user == null) {
            throw new BadRequestException(ILLEGAL_LOGIN_TYPE);
        }
        // 6.封装返回
        LoginUserDTO userDTO = new LoginUserDTO();
        userDTO.setUserId(user.getId());
        userDTO.setRoleId(handleRoleId(user));
        return userDTO;
    }

    /**
     * 手机号注册
     */
    public User phoneRegister(UserRegisterDTO dto) {
        // 1.校验验证码
        codeService.verifyCode(dto.getCellPhone(), dto.getCode());
        // 2.根据手机号查询
        User one = lambdaQuery().eq(User::getCellPhone, dto.getCellPhone()).one();
        if (one != null) {
            throw new BadRequestException("手机号已经被注册");
        }
        User user = BeanUtils.copyBean(dto, User.class);
        user.setStatus(1);
//        user.setPassword(passwordEncoder.encode(dto.getPassword()));
        user.setPassword("123456");
        user.setCreateTime(LocalDateTime.now());
        user.setUpdateTime(LocalDateTime.now());
        user.setCreater(UserContext.getUser());
        user.setUpdater(UserContext.getUser());
        user.setUsername(dto.getCellPhone());
        user.setNickName(dto.getCellPhone());
        user.setImage("https://suppliermanage.oss-cn-shanghai.aliyuncs.com/gig/img/woman.png");
        save(user);
        return user;
    }

    /**
     * 实名认证
     */
    public void verifyCode(VerifyCodeDTO dto, Long userId) {
        // 验证身份证
        if (StringUtils.isNotBlank(dto.getIdentityCardId())) {
            if (!aliyunIdCardVerifier.verifyIdentityCard(dto.getIdentityCardId(), dto.getName())) {
                throw new RuntimeException("身份证验证失败");
            }
        } else {
            throw new RuntimeException("身份证号码不能为空");
        }
        int sex = 0;
        String image = "https://suppliermanage.oss-cn-shanghai.aliyuncs.com/gig/img/woman.png";
        if(!sex(dto.getIdentityCardId())){
            sex = 1;
            image = "https://suppliermanage.oss-cn-shanghai.aliyuncs.com/gig/img/man.png";
        }
        lambdaUpdate()
                .eq(User::getId, userId)
                .set(User::getNickName, dto.getName())
                .set(User::getIdentityCardId, dto.getIdentityCardId())
                .set(User::getSex, sex)
                .set(User::getImage, image)
                .update();
    }
    private boolean sex(String getIdentityCardId) {
        char[] charArray = getIdentityCardId.toCharArray();
        return charArray[16] % 2 == 0;
    }
    /**
     * 修改登录密码
     */
    public void updatePassword(UpdatePasswordDTO dto, Long userId) {
        User user = getById(userId);
        if (user == null) {
            throw new BadRequestException(USER_ID_NOT_EXISTS);
        }
        // 1.校验旧密码
        if (!user.getPassword().equals(dto.getOldPassword())) {
            throw new BadRequestException("密码错误");
        }
        // 2.修改密码
        user.setPassword(dto.getNewPassword());
        updateById(user);
    }

    /**
     * 绑定邮箱
     */
    public void updateEmail(UpdateEmailDTO dto, Long userId) {
        // 从Redis中获取验证码
        String storedCode = stringRedisTemplate.opsForValue().get("verificationCode:" + dto.getEmail());
        // 验证码校验
        if (storedCode == null || !storedCode.equals(dto.getCode())) {
            throw new RuntimeException("验证码错误或已过期");
        }
        // 验证码正确，清空Redis中的验证码
        stringRedisTemplate.delete("verificationCode:" + dto.getEmail());
        lambdaUpdate()
                .eq(User::getId, userId)
                .set(User::getEmail, dto.getEmail())
                .update();
    }

    /**
     * 统计用户任务数量
     */
    public UserTaskDTO getStats(Long userId) {
        List<UserTask> userTaskList = userTaskService.lambdaQuery().eq(UserTask::getUserId, userId).list();

        // 检查 userTaskList 是否为空
        if (userTaskList == null || userTaskList.isEmpty()) {
            return new UserTaskDTO();
        }

        UserTaskDTO userTaskDTO = new UserTaskDTO();
        userTaskDTO.setOngoingTasks(0);
        userTaskDTO.setSettledTasks(0);

        Map<Integer, TaskMonth> tasks2024 = new HashMap<>();
        for (int i = 1; i <= 12; i++) {
            TaskMonth taskMonth = new TaskMonth();
            taskMonth.setTasksReceived(0);
            taskMonth.setTasksCompleted(0);
            tasks2024.put(i, taskMonth);
        }

        for (UserTask userTask : userTaskList) {
            // 获取任务创建月份
            LocalDateTime taskDateTime = userTask.getCreateTime();
            int month = taskDateTime.getMonthValue();

            if (userTask.getTaskStatus() == 2) {
                userTaskDTO.setOngoingTasks(userTaskDTO.getOngoingTasks() + 1);
                tasks2024.get(month).setTasksReceived(tasks2024.get(month).getTasksReceived() + 1);
            } else if (userTask.getTaskStatus() == 3) {
                userTaskDTO.setSettledTasks(userTaskDTO.getSettledTasks() + 1);
                tasks2024.get(month).setTasksReceived(tasks2024.get(month).getTasksReceived() + 1);
                tasks2024.get(month).setTasksCompleted(tasks2024.get(month).getTasksCompleted() + 1);
            }
        }
        userTaskDTO.setClaimedTasks(userTaskDTO.getOngoingTasks() + userTaskDTO.getSettledTasks());
        userTaskDTO.setTasks2024(tasks2024);

        List<Long> ids = userTaskList.stream()
                .filter(userTask -> userTask.getTaskStatus() == 3)
                .map(UserTask::getTaskId)
                .collect(Collectors.toList());

        // 检查 ids 是否为空
        if (ids.isEmpty()) {
            userTaskDTO.setTaskDetails(new ArrayList<>());
            return userTaskDTO;
        }

        List<NewTaskDTO> newTaskDTOS = taskClient.queryTaskById(ids);

        // 检查 newTaskDTOS 是否为空
        if (newTaskDTOS == null || newTaskDTOS.isEmpty()) {
            userTaskDTO.setTaskDetails(new ArrayList<>());
        } else {
            List<TaskDetail> taskDetails = BeanUtils.copyList(newTaskDTOS, TaskDetail.class);
            userTaskDTO.setTaskDetails(taskDetails);
        }

        return userTaskDTO;
    }

    /**
     * 生成微信登录二维码
     */
    public WeChatDTO wechatLogin() throws UnsupportedEncodingException {
        WeChatDTO weChatDTO = new WeChatDTO();
        String redirectUri = "https://labelx.stellvoy.com/us/users/wechat/code";
        String encodedRedirectUri = URLEncoder.encode(redirectUri, StandardCharsets.UTF_8.toString());
        String state = UUID.randomUUID().toString();
        weChatDTO.setRedirectUri(encodedRedirectUri);
        weChatDTO.setState(state);
        weChatDTO.setAppid(appId);
        stringRedisTemplate.opsForValue().set("weChatCode:" + state, state, 5, TimeUnit.MINUTES);
        return weChatDTO;
    }

    /**
     * 处理微信回调，接收 code
     */
    public void wechatCallback(String code, String state) {
        // 从Redis中获取state
        String storedCode = stringRedisTemplate.opsForValue().get("weChatCode:" + state);
        // state校验
        if (storedCode == null || !storedCode.equals(state)) {
            throw new RuntimeException("state错误或已过期");
        }
        // 使用 code 获取 access_token 和 openid
        String accessTokenUrl = "https://api.weixin.qq.com/sns/oauth2/access_token?appid=" + appId +
                "&secret=" + appSecret +
                "&code=" + code +
                "&grant_type=authorization_code";
        RestTemplate restTemplate = new RestTemplate();
        restTemplate.getMessageConverters().add(0, new StringHttpMessageConverter(StandardCharsets.UTF_8));
        ResponseEntity<String> accessTokenResponse = restTemplate.getForEntity(accessTokenUrl, String.class);
        JSONObject accessTokenJson = JSON.parseObject(accessTokenResponse.getBody());
        String accessToken = accessTokenJson.getString("access_token");
        String openid = accessTokenJson.getString("openid");
        // 使用 access_token 和 openid 获取用户信息
        String userInfoUrl = "https://api.weixin.qq.com/sns/userinfo?access_token=" + accessToken +
                "&openid=" + openid +
                "&lang=zh_CN";
        ResponseEntity<String> userInfoResponse = restTemplate.getForEntity(userInfoUrl, String.class);
        JSONObject userInfoJson = JSON.parseObject(userInfoResponse.getBody());

        // 处理用户信息，例如登录或注册用户
        String nickname = userInfoJson.getString("nickname");
        String headimgurl = userInfoJson.getString("headimgurl");
        User user = lambdaQuery().eq(User::getOpenId, openid).one();
        if(user == null){
            user = new User();
            user.setNickName(nickname);
            user.setImage(headimgurl);
            user.setOpenId(openid);
            user.setPassword("123456");
            save(user);
            user = lambdaQuery().eq(User::getOpenId, openid).one();
        }
        stringRedisTemplate.opsForValue().set("weChatCode:" + state, openid, 5, TimeUnit.MINUTES);
        stringRedisTemplate.opsForValue().set("userId:" + state, user.getId().toString(), 5, TimeUnit.MINUTES);
    }

    /**
     * 处理前端轮询
     */
    public CheckLoginDTO checkLoginStatus(String state) {
        // 从Redis中获取state
        String storedCode = stringRedisTemplate.opsForValue().get("weChatCode:" + state);
        // state校验
        if (storedCode == null) {
            throw new RuntimeException("state错误或已失效");
        }
        User one = lambdaQuery().eq(User::getOpenId, storedCode).one();
        if (one == null) {
            throw new RuntimeException("用户未登录");
        }
        LoginFormDTO loginDTO = new LoginFormDTO();
        loginDTO.setType(3);
        loginDTO.setPassword("123456");
        String userId = stringRedisTemplate.opsForValue().get("userId:" + state);
        loginDTO.setUsername(userId);
        String token = authClient.loginByPw(loginDTO);
        CheckLoginDTO checkLoginDTO = new CheckLoginDTO();
        checkLoginDTO.setToken(token);
        checkLoginDTO.setUserId(one.getId());
        if(one.getCellPhone()==null){
            checkLoginDTO.setCellPhone("未绑定手机号");
        }
        checkLoginDTO.setCellPhone(one.getCellPhone());
        return checkLoginDTO;
    }

    /**
     * 验证验证码
     */
    public void weChatVerifyCode(WeChatVerifyCodeDTO dto, Long userId) {
        codeService.verifyCode(dto.getPhone(), dto.getCode());
        lambdaUpdate().eq(User::getId, userId)
                .set(User::getCellPhone, dto.getPhone())
                .set(User::getUsername, dto.getPhone())
                .update();
    }

    public User loginByPw(LoginFormDTO loginDTO) {
        // 1.数据校验
        String cellPhone = loginDTO.getCellPhone();
        if (StrUtil.isBlank(cellPhone)) {
            throw new BadRequestException(INVALID_UN);
        }
        // 2.根据手机号查询
        User user = lambdaQuery()
                .eq(StrUtil.isNotBlank(cellPhone), User::getCellPhone, cellPhone)
                .one();
        AssertUtils.isNotNull(user, INVALID_UN_OR_PW);
        // 3.校验是否禁用
        if (user.getStatus() == 0) {
            throw new ForbiddenException(USER_FROZEN);
        }
        //4.校验密码
//        if (!passwordEncoder.matches(loginDTO.getPassword(), user.getPassword())) {
//            throw new BadRequestException(INVALID_UN_OR_PW);
//        }
        if (!loginDTO.getPassword().equals(user.getPassword())){
            throw new BadRequestException(INVALID_UN_OR_PW);
        }

        return user;
    }

    private Long handleRoleId(User user) {
        return 0L;
    }

    public User loginByVerifyCode(String phone, String code) {
        User user = lambdaQuery().eq(User::getCellPhone, phone).one();
        if (user == null) {
            UserRegisterDTO URDTO = new UserRegisterDTO();
            URDTO.setCode(code);
            URDTO.setCellPhone(phone);
            return phoneRegister(URDTO);
        }
        // 校验验证码
        codeService.verifyCode(phone, code);
        // 校验是否禁用
        if (user.getStatus() == 0) {
            throw new ForbiddenException(USER_FROZEN);
        }
        return user;
    }
}
